/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "roccat_color_custom_button.h"
#include "i18n-lib.h"

#define ROCCAT_COLOR_CUSTOM_BUTTON_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), ROCCAT_COLOR_CUSTOM_BUTTON_TYPE, RoccatColorCustomButtonPrivate))

struct _RoccatColorCustomButtonPrivate {
	GtkDialog *custom_dialog;
	GdkColor actual_color;
};

G_DEFINE_TYPE(RoccatColorCustomButton, roccat_color_custom_button, ROCCAT_COLOR_BUTTON_TYPE);

static void button_clicked_cb(RoccatColorCustomButton *color_custom_button, gpointer user_data) {
	RoccatColorCustomButtonPrivate *priv = color_custom_button->priv;
	GtkColorSelection *color_selection;
	GdkColor color;
	
	color_selection = GTK_COLOR_SELECTION(gtk_color_selection_dialog_get_color_selection(GTK_COLOR_SELECTION_DIALOG(priv->custom_dialog)));
	gtk_color_selection_set_current_color(color_selection, &priv->actual_color);
	gtk_color_selection_set_previous_color(color_selection, &priv->actual_color);
	if (gtk_dialog_run(priv->custom_dialog) == GTK_RESPONSE_OK) {
		gtk_color_selection_get_current_color(color_selection, &color);
		roccat_color_custom_button_set_color(color_custom_button, &color);
	}
	gtk_widget_hide(GTK_WIDGET(priv->custom_dialog));
}

GtkWidget *roccat_color_custom_button_new(void) {
	return GTK_WIDGET(g_object_new(ROCCAT_COLOR_CUSTOM_BUTTON_TYPE, NULL));
}

static void update(RoccatColorCustomButton *color_custom_button) {
	roccat_color_button_set_color(ROCCAT_COLOR_BUTTON(color_custom_button), &color_custom_button->priv->actual_color);
}

void roccat_color_custom_button_get_color(RoccatColorCustomButton *color_custom_button, GdkColor *color) {
	*color = color_custom_button->priv->actual_color;
}

void roccat_color_custom_button_set_color(RoccatColorCustomButton *color_custom_button, GdkColor const *color) {
	RoccatColorCustomButtonPrivate *priv = color_custom_button->priv;

	if (!gdk_color_equal(color, &priv->actual_color)) {
		priv->actual_color = *color;
		g_signal_emit_by_name((gpointer)color_custom_button, "value-changed");
	}

	update(color_custom_button);
}

static void roccat_color_custom_button_init(RoccatColorCustomButton *color_custom_button) {
	RoccatColorCustomButtonPrivate *priv = ROCCAT_COLOR_CUSTOM_BUTTON_GET_PRIVATE(color_custom_button);

	color_custom_button->priv = priv;
	priv->custom_dialog = GTK_DIALOG(gtk_color_selection_dialog_new(_("Choose color")));	
	g_signal_connect(G_OBJECT(color_custom_button), "clicked", G_CALLBACK(button_clicked_cb), NULL);
}

static void roccat_color_custom_button_finalize(GObject *object) {
	RoccatColorCustomButtonPrivate *priv = ROCCAT_COLOR_CUSTOM_BUTTON(object)->priv;
	gtk_widget_destroy(GTK_WIDGET(priv->custom_dialog));
	G_OBJECT_CLASS(roccat_color_custom_button_parent_class)->finalize(object);
}

static void roccat_color_custom_button_class_init(RoccatColorCustomButtonClass *klass) {
	GObjectClass *gobject_class = (GObjectClass*)klass;

	gobject_class->finalize = roccat_color_custom_button_finalize;

	g_type_class_add_private(klass, sizeof(RoccatColorCustomButtonPrivate));

	g_signal_new("value-changed",
			G_TYPE_FROM_CLASS(klass),
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			G_STRUCT_OFFSET(RoccatColorCustomButtonClass, value_changed),
			NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
}
